package security;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.*;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.interfaces.DSAKey;
import java.util.Enumeration;
import java.util.Vector;

/**
 * DocJava, Inc. User: lyon Date: May 20, 2004
 * Time: 6:35:50 AM
 */
public class KeyUtils {
    public static void main(String[] args) {
        testGetAlias();
    }


    public KeyUtils() {
    }

    public static void printProviders() {
        System.out.println("provider list:");
        Provider p[] = Security.getProviders();
        for (int i = 0; i < p.length; i++)
            printProvider(p[i]);
    }

    private static void printProvider(
            final Provider provider) {
        System.out.println(
                "Name:" + provider.getName());
        System.out.println(
                "Provider:" + provider);
        System.out.println(
                "Info:" + provider.getInfo());
        System.out.println("algorithm:" +
                           provider.getProperty(
                                   "algorithm"));

        System.out.println("-----");
    }

    public static void printKey(Key k) {
        if (k instanceof DSAKey) {
            System.out.println("key is DSA");
            System.out.println("P value is " +
                               ((DSAKey) k).getParams()
                               .getP());
        } else {
            System.out.println("key is NOT DSA");
            System.out.println(k);
        }
    }


    public static Certificate getCertificate() {
        String password = getPassword();
        String alias = getAlias();
        return
                getCertificate(alias,
                               password);
    }

    public static String getPassword() {
        String password = futils.In.getString(
                "please enter keystore password");
        return password;
    }

    public static String getAlias() {
        String alias = futils.In.getString(
                "please enter keystore alias");
        return alias;
    }

    public static Certificate getCertificate(
            String alias, String password) {
        try {
            // Load the keystore in the user's home directory
            FileInputStream is = new FileInputStream(System.getProperty(
                    "user.home")
                                                     +
                                                     File.separatorChar +
                                                     ".keystore");

            KeyStore keystore = KeyStore.getInstance(
                    KeyStore.getDefaultType());
            keystore.load(is,
                          password.toCharArray());

            // Get certificate

            return keystore.getCertificate(alias);
        } catch (KeyStoreException e) {
            e.printStackTrace();
        } catch (CertificateException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static KeyPair getKeyPair(
            KeyStore keystore,
            String alias,
            String password) {
        try {
            // Get private key
            Key key = keystore.getKey(alias,
                                      password.toCharArray());
            if (key instanceof PrivateKey) {
                // Get certificate of public key
                java.security.cert.Certificate cert = keystore.getCertificate(
                        alias);

                // Get public key
                PublicKey publicKey = cert.getPublicKey();

                // Return a key pair
                return new KeyPair(publicKey,
                                   (PrivateKey) key);
            }
        } catch (UnrecoverableKeyException e) {
        } catch (NoSuchAlgorithmException e) {
        } catch (KeyStoreException e) {
        }
        return null;
    }

    public static KeyStore getKeyStore() {
        return getKeyStore(getPassword());
    }

    public static KeyStore getKeyStore(
            String password) {
        File ksf = getKeystoreFile();
        return getKeyStore(ksf, password);
    }

    public static KeyStore getKeyStore(
            File keyStoreFile, String password) {
        KeyStore keystore = null;
        try {

            FileInputStream is = new FileInputStream(
                    keyStoreFile);
            keystore =
            KeyStore.getInstance(
                    KeyStore.getDefaultType());
            keystore.load(is,
                          password.toCharArray());
            is.close();
        } catch (KeyStoreException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (CertificateException e) {
            e.printStackTrace();
        }
        return keystore;
    }

    public static String[] getAliasArray(
            String password) {


        KeyStore keystore = getKeyStore(password);


        Enumeration enum = null;
        try {
            // List the aliases
            enum = keystore.aliases();
        } catch (KeyStoreException e) {
            e.printStackTrace();
        }
        Vector v = new Vector();
        for (; enum.hasMoreElements();) {
            String alias = (String) enum.nextElement();

            v.addElement(alias);

        }
        String s[] = new String[v.size()];
        v.copyInto(s);

        return s;


    }

    private static File getKeystoreFile() {
        File file = new File(System.getProperty(
                "user.home") +
                             File.separatorChar +
                             ".keystore");
        return file;
    }


    private static void testGetAlias() {
        String password = futils.In.getString(
                "please enter keystore password");
        String s[] = getAliasArray(password);
        for (int i = 0; i < s.length; i++)
            System.out.println("alias:" + s[i]);
    }

    public static void print(Certificate c) {
        System.out.println("-----" +
                           "\n" +
                           c.getClass().toString() +
                           "\n" + c);
    }

    public static void print(Certificate[] c) {
        for (int i = 0; i < c.length; i++)
            print(c[i]);
    }

}
